@@ -3,6 +3,9 @@ from fixieai.agents import AgentResponse
 from fixieai.agents import CodeShotAgent
 from fixieai.agents import Embed
 from fixieai.agents import Message
+from fixieai.agents import OAuthHandler
+from fixieai.agents import OAuthParams
+from fixieai.agents import UserStorage
 from fixieai.client import FixieClient
 from fixieai.client import get_agents
 from fixieai.client import get_client
@@ -15,6 +18,9 @@ __all__ = [
     "Embed",
     "Message",
     "CodeShotAgent",
+    "OAuthParams",
+    "OAuthHandler",
+    "UserStorage",
     "FixieClient",
     "get_agents",
     "get_client",
