@@ -8,8 +8,6 @@ from typing import Dict, Optional
 
 from pydantic import dataclasses as pydantic_dataclasses
 
-from fixieai.agents import user_storage
-
 
 @pydantic_dataclasses.dataclass
 class Embed:
@@ -47,17 +45,6 @@ class AgentQuery:
     # can be tied back to the original user.
     access_token: Optional[str] = None
 
-    def user_storage(self, agent_id: str) -> user_storage.UserStorage:
-        """Returns a persistent, user-specific, dict-like memory storage.
-
-        Args:
-            agent_id: The agent who's asking for it.
-
-        TODO(hessam): Remove agent_id from the signature once agent_id is included in
-         access_token.
-        """
-        return user_storage.UserStorage(self, agent_id)
-
 
 @pydantic_dataclasses.dataclass
 class AgentResponse:
